@@ -204,6 +204,14 @@ artwork_rescale(struct evbuffer *evbuf, AVFormatContext *src_ctx, int s, int out
       return -1;
     }
 
+  if (src->pix_fmt < 0)
+    {
+      DPRINTF(E_LOG, L_ART, "Unknown pixel format for artwork %s\n", src_ctx->filename);
+
+      ret = -1;
+      goto out_close_src;
+    }
+
   /* Set up output */
   dst_fmt = av_guess_format("image2", NULL, NULL);
   if (!dst_fmt)
@@ -684,6 +692,7 @@ artwork_get_player_image(struct evbuffer *evbuf, char *path)
   struct keyval *kv;
   const char *content_type;
   char *url;
+  char *ext;
   int format;
   int id;
   int len;
@@ -704,6 +713,12 @@ artwork_get_player_image(struct evbuffer *evbuf, char *path)
   if ((len < 14) || (len > PATH_MAX)) // Can't be shorter than http://a/1.jpg
     goto out_url;
 
+  ext = strrchr(url, '.');
+  if (!ext)
+    goto out_url;
+  if ((strcmp(ext, ".jpg") != 0) && (strcmp(ext, ".png") != 0))
+    goto out_url;
+
   cache_artwork_read(evbuf, url, &format);
   if (format > 0)
     goto out_url;
