@@ -2,9 +2,6 @@
 #ifndef __ARTWORK_H__
 #define __ARTWORK_H__
 
-#define ART_CAN_PNG     (1 << 0)
-#define ART_CAN_JPEG    (1 << 1)
-
 #define ART_FMT_PNG     1
 #define ART_FMT_JPEG    2
 
@@ -16,10 +13,14 @@
 
 /* Get artwork for individual track */
 int
-artwork_get_item(int id, int max_w, int max_h, int format, struct evbuffer *evbuf);
+artwork_get_item(struct evbuffer *evbuf, int id, int max_w, int max_h);
 
 /* Get artwork for album or artist */
 int
-artwork_get_group(int id, int max_w, int max_h, int format, struct evbuffer *evbuf);
+artwork_get_group(struct evbuffer *evbuf, int id, int max_w, int max_h);
+
+/* Checks if the file is an artwork file */
+int
+artwork_file_is_artwork(const char *filename);
 
 #endif /* !__ARTWORK_H__ */
