@@ -23,6 +23,10 @@
 #include <string.h>
 #include <stdint.h>
 
+#ifndef HAVE_LIBEVENT2
+# define evbuffer_get_length(x) (x)->off
+#endif
+
 #include "db.h"
 #include "misc.h"
 #include "logger.h"
@@ -43,7 +47,7 @@ dmap_get_fields_table(int *nfields)
 
 
 void
-dmap_add_container(struct evbuffer *evbuf, char *tag, int len)
+dmap_add_container(struct evbuffer *evbuf, const char *tag, int len)
 {
   unsigned char buf[4];
 
@@ -59,7 +63,7 @@ dmap_add_container(struct evbuffer *evbuf, char *tag, int len)
 }
 
 void
-dmap_add_long(struct evbuffer *evbuf, char *tag, int64_t val)
+dmap_add_long(struct evbuffer *evbuf, const char *tag, int64_t val)
 {
   unsigned char buf[12];
 
@@ -85,7 +89,7 @@ dmap_add_long(struct evbuffer *evbuf, char *tag, int64_t val)
 }
 
 void
-dmap_add_int(struct evbuffer *evbuf, char *tag, int val)
+dmap_add_int(struct evbuffer *evbuf, const char *tag, int val)
 {
   unsigned char buf[8];
 
@@ -107,7 +111,7 @@ dmap_add_int(struct evbuffer *evbuf, char *tag, int val)
 }
 
 void
-dmap_add_short(struct evbuffer *evbuf, char *tag, short val)
+dmap_add_short(struct evbuffer *evbuf, const char *tag, short val)
 {
   unsigned char buf[6];
 
@@ -127,7 +131,7 @@ dmap_add_short(struct evbuffer *evbuf, char *tag, short val)
 }
 
 void
-dmap_add_char(struct evbuffer *evbuf, char *tag, char val)
+dmap_add_char(struct evbuffer *evbuf, const char *tag, char val)
 {
   unsigned char buf[5];
 
@@ -146,7 +150,7 @@ dmap_add_char(struct evbuffer *evbuf, char *tag, char val)
 }
 
 void
-dmap_add_literal(struct evbuffer *evbuf, char *tag, char *str, int len)
+dmap_add_literal(struct evbuffer *evbuf, const char *tag, char *str, int len)
 {
   char buf[4];
 
@@ -179,7 +183,7 @@ dmap_add_raw_uint32(struct evbuffer *evbuf, uint32_t val)
 }
 
 void
-dmap_add_string(struct evbuffer *evbuf, char *tag, const char *str)
+dmap_add_string(struct evbuffer *evbuf, const char *tag, const char *str)
 {
   unsigned char buf[4];
   int len;
@@ -341,12 +345,15 @@ dmap_add_field(struct evbuffer *evbuf, const struct dmap_field *df, char *strval
 
 
 void
-dmap_send_error(struct evhttp_request *req, char *container, char *errmsg)
+dmap_send_error(struct evhttp_request *req, const char *container, const char *errmsg)
 {
   struct evbuffer *evbuf;
   int len;
   int ret;
 
+  if (!req)
+    return;
+
   evbuf = evbuffer_new();
   if (!evbuf)
     {
@@ -526,7 +533,7 @@ dmap_encode_file_metadata(struct evbuffer *songlist, struct evbuffer *song, stru
   if (want_asdk)
     val += 9;
 
-  dmap_add_container(songlist, "mlit", EVBUFFER_LENGTH(song) + val);
+  dmap_add_container(songlist, "mlit", evbuffer_get_length(song) + val);
 
   /* Prepend mikd & asdk if needed */
   if (want_mikd)
