@@ -699,6 +699,13 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
   mfi->time_modified = mtime;
   mfi->file_size = size;
 
+  if (type & F_SCAN_TYPE_COMPILATION)
+    mfi->compilation = 1;
+  if (type & F_SCAN_TYPE_PODCAST)
+    mfi->media_kind = MEDIA_KIND_PODCAST; /* podcast */
+  if (type & F_SCAN_TYPE_AUDIOBOOK)
+    mfi->media_kind = MEDIA_KIND_AUDIOBOOK; /* audiobook */
+
   if (type & F_SCAN_TYPE_FILE)
     {
       mfi->data_kind = DATA_KIND_FILE;
@@ -706,7 +713,7 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
     }
   else if (type & F_SCAN_TYPE_URL)
     {
-      mfi->data_kind = DATA_KIND_URL;
+      mfi->data_kind = DATA_KIND_HTTP;
       ret = scan_metadata_ffmpeg(path, mfi);
       if (ret < 0)
 	{
@@ -742,13 +749,6 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
       goto out;
     }
 
-  if (type & F_SCAN_TYPE_COMPILATION)
-    mfi->compilation = 1;
-  if (type & F_SCAN_TYPE_PODCAST)
-    mfi->media_kind = MEDIA_KIND_PODCAST; /* podcast */
-  if (type & F_SCAN_TYPE_AUDIOBOOK)
-    mfi->media_kind = MEDIA_KIND_AUDIOBOOK; /* audiobook */
-
   if (!mfi->item_kind)
     mfi->item_kind = 2; /* music */
   if (!mfi->media_kind)
