@@ -31,9 +31,21 @@ scan_metadata_icy(char *url, struct media_file_info *mfi);
 void
 scan_playlist(char *file, time_t mtime);
 
+void
+scan_smartpl(char *file, time_t mtime);
+
 #ifdef ITUNES
 void
 scan_itunes_itml(char *file);
 #endif
 
+void
+filescanner_trigger_initscan(void);
+
+void
+filescanner_trigger_fullrescan(void);
+
+int
+filescanner_scanning(void);
+
 #endif /* !__FILESCANNER_H__ */
