@@ -39,17 +39,6 @@
 #include "misc.h"
 #include "http.h"
 
-
-/* Legacy format-specific scanners */
-extern int scan_get_wmainfo(char *filename, struct media_file_info *pmp3);
-#ifdef FLAC
-extern int scan_get_flacinfo(char *filename, struct media_file_info *pmp3);
-#endif
-#ifdef MUSEPACK
-extern int scan_get_mpcinfo(char *filename, struct media_file_info *pmp3);
-#endif
-
-
 /* Mapping between the metadata name(s) and the offset
  * of the equivalent metadata field in struct media_file_info */
 struct metadata_map {
@@ -346,14 +335,14 @@ scan_metadata_ffmpeg(char *file, struct media_file_info *mfi)
 #if LIBAVFORMAT_VERSION_MAJOR >= 54 || (LIBAVFORMAT_VERSION_MAJOR == 53 && LIBAVFORMAT_VERSION_MINOR >= 3)
 # ifndef HAVE_FFMPEG
   // Without this, libav is slow to probe some internet streams
-  if (mfi->data_kind == DATA_KIND_URL)
+  if (mfi->data_kind == DATA_KIND_HTTP)
     {
       ctx = avformat_alloc_context();
       ctx->probesize = 64000;
     }
 # endif
 
-  if (mfi->data_kind == DATA_KIND_URL)
+  if (mfi->data_kind == DATA_KIND_HTTP)
     {
       free(path);
       ret = http_stream_setup(&path, file);
@@ -438,6 +427,10 @@ scan_metadata_ffmpeg(char *file, struct media_file_info *mfi)
 		break;
 	      }
 #endif
+	    // We treat these as audio no matter what
+	    if (mfi->compilation || (mfi->media_kind & (MEDIA_KIND_PODCAST | MEDIA_KIND_AUDIOBOOK)))
+	      break;
+
 	    if (!video_stream)
 	      {
 		DPRINTF(E_DBG, L_SCAN, "File has video (stream %d)\n", i);
@@ -492,8 +485,8 @@ scan_metadata_ffmpeg(char *file, struct media_file_info *mfi)
 
   DPRINTF(E_DBG, L_SCAN, "Duration %d ms, bitrate %d kbps\n", mfi->song_length, mfi->bitrate);
 
-  /* Try to extract ICY metadata if url/stream */
-  if (mfi->data_kind == DATA_KIND_URL)
+  /* Try to extract ICY metadata if http stream */
+  if (mfi->data_kind == DATA_KIND_HTTP)
     {
       icy_metadata = http_icy_metadata_get(ctx, 0);
       if (icy_metadata && icy_metadata->name)
@@ -769,37 +762,7 @@ scan_metadata_ffmpeg(char *file, struct media_file_info *mfi)
 #endif
 
   if (mdcount == 0)
-    {
-#if LIBAVFORMAT_VERSION_MAJOR < 54 || (LIBAVFORMAT_VERSION_MAJOR == 54 && LIBAVFORMAT_VERSION_MINOR < 35)
-      /* ffmpeg doesn't support FLAC nor Musepack metadata,
-       * and is buggy for some WMA variants, so fall back to the
-       * legacy format-specific parsers until it gets fixed */
-      if ((codec_id == CODEC_ID_WMAPRO)
-	  || (codec_id == CODEC_ID_WMAVOICE)
-	  || (codec_id == CODEC_ID_WMALOSSLESS))
-	{
-	  DPRINTF(E_WARN, L_SCAN, "Falling back to legacy WMA scanner\n");
-	  return (scan_get_wmainfo(file, mfi) ? 0 : -1);
-	}
-#ifdef FLAC
-      else if (codec_id == CODEC_ID_FLAC)
-	{
-	  DPRINTF(E_WARN, L_SCAN, "Falling back to legacy FLAC scanner\n");
-	  return (scan_get_flacinfo(file, mfi) ? 0 : -1);
-	}
-#endif /* FLAC */
-#ifdef MUSEPACK
-      else if ((codec_id == CODEC_ID_MUSEPACK7)
-	       || (codec_id == CODEC_ID_MUSEPACK8))
-	{
-	  DPRINTF(E_WARN, L_SCAN, "Falling back to legacy Musepack scanner\n");
-	  return (scan_get_mpcinfo(file, mfi) ? 0 : -1);
-	}
-#endif /* MUSEPACK */
-      else
-#endif /* LIBAVFORMAT */
-	DPRINTF(E_WARN, L_SCAN, "ffmpeg/libav could not extract any metadata\n");
-    }
+    DPRINTF(E_WARN, L_SCAN, "ffmpeg/libav could not extract any metadata\n");
 
   /* Just in case there's no title set ... */
   if (mfi->title == NULL)
