@@ -204,10 +204,11 @@ stream_chunk_xcode_cb(int fd, short event, void *arg)
   struct timeval tv;
   int xcoded;
   int ret;
+  int dummy;
 
   st = (struct stream_ctx *)arg;
 
-  xcoded = transcode(st->xcode, st->evbuf, STREAM_CHUNK_SIZE);
+  xcoded = transcode(st->xcode, st->evbuf, STREAM_CHUNK_SIZE, &dummy);
   if (xcoded <= 0)
     {
       if (xcoded == 0)
@@ -414,7 +415,7 @@ httpd_stream_file(struct evhttp_request *req, int id)
       return;
     }
 
-  if (mfi->data_kind != 0)
+  if (mfi->data_kind != DATA_KIND_FILE)
     {
       evhttp_send_error(req, 500, "Cannot stream radio station");
 
@@ -1007,7 +1008,7 @@ httpd_gen_cb(struct evhttp_request *req, void *arg)
   ptr = strchr(uri, '?');
   if (ptr)
     {
-      DPRINTF(E_DBG, L_HTTPD, "Found query string\n");
+      DPRINTF(E_SPAM, L_HTTPD, "Found query string\n");
 
       *ptr = '\0';
     }
@@ -1272,14 +1273,12 @@ httpd_basic_auth(struct evhttp_request *req, char *user, char *passwd, char *rea
 int
 httpd_init(void)
 {
+  const char *addr;
   unsigned short port;
-  int v6enabled;
   int ret;
 
   httpd_exit = 0;
 
-  v6enabled = cfg_getbool(cfg_getsec(cfg, "general"), "ipv6");
-
   evbase_httpd = event_base_new();
   if (!evbase_httpd)
     {
@@ -1350,27 +1349,21 @@ httpd_init(void)
       goto evhttp_fail;
     }
 
+  if (cfg_getbool(cfg_getsec(cfg, "general"), "ipv6"))
+    addr = "::";
+  else
+    addr = "0.0.0.0";
+
   port = cfg_getint(cfg_getsec(cfg, "library"), "port");
 
-  /* We are binding v6 and v4 separately, and we allow v6 to fail
-   * as IPv6 might not be supported on the system.
-   * We still warn about the failure, in case there's another issue.
-   */
-  ret = evhttp_bind_socket(evhttpd, "0.0.0.0", port);
+  ret = evhttp_bind_socket(evhttpd, addr, port);
   if (ret < 0)
     {
-      DPRINTF(E_FATAL, L_HTTPD, "Could not bind INADDR_ANY:%d\n", port);
+      DPRINTF(E_FATAL, L_HTTPD, "Could not bind %s:%d\n", addr, port);
 
       goto bind_fail;
     }
 
-  if (v6enabled)
-    {
-      ret = evhttp_bind_socket(evhttpd, "::", port);
-      if (ret < 0)
-	DPRINTF(E_WARN, L_HTTPD, "Could not bind IN6ADDR_ANY:%d (that's OK)\n", port);
-    }
-
   evhttp_set_gencb(evhttpd, httpd_gen_cb, NULL);
 
   ret = pthread_create(&tid_httpd, NULL, httpd, NULL);
