@@ -2,13 +2,10 @@
 #ifndef __LASTFM_H__
 #define __LASTFM_H__
 
-void
+int
 lastfm_login(char *path);
 
 int
 lastfm_scrobble(int id);
 
-void
-lastfm_deinit(void);
-
 #endif /* !__LASTFM_H__ */
