@@ -119,6 +119,10 @@ safe_hextou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
+  /* A hex shall begin with 0x */
+  if (strncmp(str, "0x", 2) != 0)
+    return safe_atou32(str, val);
+
   errno = 0;
   intval = strtoul(str, &end, 16);
 
@@ -482,8 +486,9 @@ m_realpath(const char *pathname)
   return ret;
 }
 
+
 char *
-unicode_fixup_string(char *str)
+unicode_fixup_string(char *str, const char *fromcode)
 {
   uint8_t *ret;
   size_t len;
@@ -506,7 +511,7 @@ unicode_fixup_string(char *str)
       return str;
     }
 
-  ret = u8_conv_from_encoding("ascii", iconveh_question_mark, str, len, NULL, NULL, &len);
+  ret = u8_strconv_from_encoding(str, fromcode, iconveh_question_mark);
   if (!ret)
     {
       DPRINTF(E_LOG, L_MISC, "Could not convert string '%s' to UTF-8: %s\n", str, strerror(errno));
