@@ -47,7 +47,11 @@
 #endif
 
 #include <event.h>
-#include "evhttp/evhttp.h"
+#ifdef HAVE_LIBEVENT2
+# include <event2/http.h>
+#else
+# include "evhttp/evhttp_compat.h"
+#endif
 
 #include <gcrypt.h>
 
@@ -410,8 +414,10 @@ static void
 pairing_request_cb(struct evhttp_request *req, void *arg)
 {
   struct remote_info *ri;
+  struct evbuffer *input_buffer;
   uint8_t *response;
   char guid[17];
+  int response_code;
   int len;
   int i;
   int ret;
@@ -419,23 +425,30 @@ pairing_request_cb(struct evhttp_request *req, void *arg)
   ri = (struct remote_info *)arg;
 
   if (!req)
-    goto cleanup;
-
-  if (req->response_code != HTTP_OK)
     {
-      DPRINTF(E_LOG, L_REMOTE, "Pairing failed with Remote %s/%s, HTTP response code %d\n", ri->pi.remote_id, ri->pi.name, req->response_code);
+      DPRINTF(E_LOG, L_REMOTE, "Empty pairing request callback\n");
 
       goto cleanup;
     }
 
-  if (EVBUFFER_LENGTH(req->input_buffer) < 8)
+  response_code = evhttp_request_get_response_code(req);
+  if (response_code != HTTP_OK)
+    {
+      DPRINTF(E_LOG, L_REMOTE, "Pairing failed with Remote %s/%s, HTTP response code %d\n", ri->pi.remote_id, ri->pi.name, response_code);
+
+      goto cleanup;
+    }
+
+  input_buffer = evhttp_request_get_input_buffer(req);
+
+  if (EVBUFFER_LENGTH(input_buffer) < 8)
     {
       DPRINTF(E_LOG, L_REMOTE, "Remote %s/%s: pairing response too short\n", ri->pi.remote_id, ri->pi.name);
 
       goto cleanup;
     }
 
-  response = EVBUFFER_DATA(req->input_buffer);
+  response = EVBUFFER_DATA(input_buffer);
 
   if ((response[0] != 'c') || (response[1] != 'm') || (response[2] != 'p') || (response[3] != 'a'))
     {
@@ -445,10 +458,10 @@ pairing_request_cb(struct evhttp_request *req, void *arg)
     }
 
   len = (response[4] << 24) | (response[5] << 16) | (response[6] << 8) | (response[7]);
-  if (EVBUFFER_LENGTH(req->input_buffer) < 8 + len)
+  if (EVBUFFER_LENGTH(input_buffer) < 8 + len)
     {
       DPRINTF(E_LOG, L_REMOTE, "Remote %s/%s: pairing response truncated (got %d expected %d)\n",
-	      ri->pi.remote_id, ri->pi.name, (int)EVBUFFER_LENGTH(req->input_buffer), len + 8);
+	      ri->pi.remote_id, ri->pi.name, (int)EVBUFFER_LENGTH(input_buffer), len + 8);
 
       goto cleanup;
     }
@@ -528,7 +541,7 @@ send_pairing_request(struct remote_info *ri, char *req_uri, int family)
 	return -1;
     }
 
-  evcon = evhttp_connection_new(address, port);
+  evcon = evhttp_connection_base_new(evbase_main, NULL, address, port);
   if (!evcon)
     {
       DPRINTF(E_LOG, L_REMOTE, "Could not create connection for pairing with %s\n", ri->pi.name);
@@ -536,8 +549,6 @@ send_pairing_request(struct remote_info *ri, char *req_uri, int family)
       return -1;
     }
 
-  evhttp_connection_set_base(evcon, evbase_main);
-
   req = evhttp_request_new(pairing_request_cb, ri);
   if (!req)
     {
@@ -554,6 +565,8 @@ send_pairing_request(struct remote_info *ri, char *req_uri, int family)
       goto request_fail;
     }
 
+  DPRINTF(E_DBG, L_REMOTE, "Pairing requested to %s\n", req_uri);
+
   ri->evcon = evcon;
 
   return 0;
@@ -748,7 +761,7 @@ touch_remote_cb(const char *name, const char *type, const char *domain, const ch
 	  return;
 	}
 
-      DPRINTF(E_DBG, L_REMOTE, "Discovered remote %s (id %s) at [%s]:%d, paircode %s\n", devname, name, address, port, paircode);
+      DPRINTF(E_LOG, L_REMOTE, "Discovered remote '%s' (id %s) at %s:%d, paircode %s\n", devname, name, address, port, paircode);
 
       /* Add the data to the list, adding the remote to the list if needed */
       pthread_mutex_lock(&remote_lck);
@@ -873,7 +886,7 @@ remote_pairing_read_pin(char *path)
       return;
     }
 
-  DPRINTF(E_DBG, L_REMOTE, "Adding Remote pin data: name '%s', pin '%s'\n", devname, pin);
+  DPRINTF(E_LOG, L_REMOTE, "Read Remote pairing data (name '%s', pin '%s') from %s\n", devname, pin, path);
 
   pthread_mutex_lock(&remote_lck);
 
