@@ -46,6 +46,9 @@
 #include "filescanner.h"
 
 
+/* How long to wait for audio (in sec) before giving up */
+#define SPOTIFY_TIMEOUT 20
+
 /* --- Types --- */
 typedef struct audio_fifo_data
 {
@@ -116,6 +119,10 @@ struct spotify_command
 // Spotify thread
 static pthread_t tid_spotify;
 
+// Used to make sure no login is attempted before the logout cb from Spotify
+static pthread_mutex_t login_lck;
+static pthread_cond_t login_cond;
+
 // Event base, pipes and events
 struct event_base *evbase_spotify;
 static int g_exit_pipe[2];
@@ -133,6 +140,8 @@ static void *g_libhandle;
 static enum spotify_state g_state;
 /* (not used) Tells which commmand is currently being processed */
 static struct spotify_command *g_cmd;
+// The global base playlist id (parent of all Spotify playlists in the db)
+static int g_base_plid;
 
 // Audio fifo
 static audio_fifo_t *g_audio_fifo;
@@ -177,6 +186,7 @@ typedef sp_error     (*fptr_sp_session_login_t)(sp_session *session, const char
 typedef sp_error     (*fptr_sp_session_relogin_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_logout_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_process_events_t)(sp_session *session, int *next_timeout);
+typedef sp_playlist* (*fptr_sp_session_starred_create_t)(sp_session *session);
 typedef sp_playlistcontainer* (*fptr_sp_session_playlistcontainer_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_player_load_t)(sp_session *session, sp_track *track);
 typedef sp_error     (*fptr_sp_session_player_unload_t)(sp_session *session);
@@ -195,6 +205,7 @@ typedef sp_error     (*fptr_sp_playlist_remove_callbacks_t)(sp_playlist *playlis
 typedef int          (*fptr_sp_playlist_num_tracks_t)(sp_playlist *playlist);
 typedef sp_track*    (*fptr_sp_playlist_track_t)(sp_playlist *playlist, int index);
 typedef bool         (*fptr_sp_playlist_is_loaded_t)(sp_playlist *playlist);
+typedef int          (*fptr_sp_playlist_track_create_time_t)(sp_playlist *playlist, int index);
 
 typedef sp_error     (*fptr_sp_track_error_t)(sp_track *track);
 typedef bool         (*fptr_sp_track_is_loaded_t)(sp_track *track);
@@ -204,6 +215,7 @@ typedef int          (*fptr_sp_track_index_t)(sp_track *track);
 typedef int          (*fptr_sp_track_disc_t)(sp_track *track);
 typedef sp_album*    (*fptr_sp_track_album_t)(sp_track *track);
 typedef sp_track_availability (*fptr_sp_track_get_availability_t)(sp_session *session, sp_track *track);
+typedef bool         (*fptr_sp_track_is_starred_t)(sp_session *session, sp_track *track);
 
 typedef sp_link*     (*fptr_sp_link_create_from_playlist_t)(sp_playlist *playlist);
 typedef sp_link*     (*fptr_sp_link_create_from_track_t)(sp_track *track, int offset);
@@ -235,6 +247,7 @@ fptr_sp_session_release_t fptr_sp_session_release;
 fptr_sp_session_login_t fptr_sp_session_login;
 fptr_sp_session_relogin_t fptr_sp_session_relogin;
 fptr_sp_session_logout_t fptr_sp_session_logout;
+fptr_sp_session_starred_create_t fptr_sp_session_starred_create;
 fptr_sp_session_playlistcontainer_t fptr_sp_session_playlistcontainer;
 fptr_sp_session_process_events_t fptr_sp_session_process_events;
 fptr_sp_session_player_load_t fptr_sp_session_player_load;
@@ -254,6 +267,7 @@ fptr_sp_playlist_remove_callbacks_t fptr_sp_playlist_remove_callbacks;
 fptr_sp_playlist_num_tracks_t fptr_sp_playlist_num_tracks;
 fptr_sp_playlist_track_t fptr_sp_playlist_track;
 fptr_sp_playlist_is_loaded_t fptr_sp_playlist_is_loaded;
+fptr_sp_playlist_track_create_time_t fptr_sp_playlist_track_create_time;
 
 fptr_sp_track_error_t fptr_sp_track_error;
 fptr_sp_track_is_loaded_t fptr_sp_track_is_loaded;
@@ -263,6 +277,7 @@ fptr_sp_track_index_t fptr_sp_track_index;
 fptr_sp_track_disc_t fptr_sp_track_disc;
 fptr_sp_track_album_t fptr_sp_track_album;
 fptr_sp_track_get_availability_t fptr_sp_track_get_availability;
+fptr_sp_track_is_starred_t fptr_sp_track_is_starred;
 
 fptr_sp_link_create_from_playlist_t fptr_sp_link_create_from_playlist;
 fptr_sp_link_create_from_track_t fptr_sp_link_create_from_track;
@@ -313,6 +328,7 @@ fptr_assign_all()
    && (fptr_sp_session_preferred_bitrate = dlsym(h, "sp_session_preferred_bitrate"))
    && (fptr_sp_playlistcontainer_add_callbacks = dlsym(h, "sp_playlistcontainer_add_callbacks"))
    && (fptr_sp_playlistcontainer_num_playlists = dlsym(h, "sp_playlistcontainer_num_playlists"))
+   && (fptr_sp_session_starred_create = dlsym(h, "sp_session_starred_create"))
    && (fptr_sp_playlistcontainer_playlist = dlsym(h, "sp_playlistcontainer_playlist"))
    && (fptr_sp_playlist_add_callbacks = dlsym(h, "sp_playlist_add_callbacks"))
    && (fptr_sp_playlist_name = dlsym(h, "sp_playlist_name"))
@@ -320,6 +336,7 @@ fptr_assign_all()
    && (fptr_sp_playlist_num_tracks = dlsym(h, "sp_playlist_num_tracks"))
    && (fptr_sp_playlist_track = dlsym(h, "sp_playlist_track"))
    && (fptr_sp_playlist_is_loaded = dlsym(h, "sp_playlist_is_loaded"))
+   && (fptr_sp_playlist_track_create_time = dlsym(h, "sp_playlist_track_create_time"))
    && (fptr_sp_track_error = dlsym(h, "sp_track_error"))
    && (fptr_sp_track_is_loaded = dlsym(h, "sp_track_is_loaded"))
    && (fptr_sp_track_name = dlsym(h, "sp_track_name"))
@@ -328,6 +345,7 @@ fptr_assign_all()
    && (fptr_sp_track_disc = dlsym(h, "sp_track_disc"))
    && (fptr_sp_track_album = dlsym(h, "sp_track_album"))
    && (fptr_sp_track_get_availability = dlsym(h, "sp_track_get_availability"))
+   && (fptr_sp_track_is_starred = dlsym(h, "sp_track_is_starred"))
    && (fptr_sp_link_create_from_playlist = dlsym(h, "sp_link_create_from_playlist"))
    && (fptr_sp_link_create_from_track = dlsym(h, "sp_link_create_from_track"))
    && (fptr_sp_link_create_from_string = dlsym(h, "sp_link_create_from_string"))
@@ -452,11 +470,25 @@ thread_exit(void)
 /*            Should only be called from within the spotify thread           */
 
 static int
-spotify_metadata_get(sp_track *track, struct media_file_info *mfi)
+spotify_metadata_get(sp_track *track, struct media_file_info *mfi, const char *pltitle, int time_added)
 {
+  cfg_t *spotify_cfg;
+  bool artist_override;
+  bool starred_artist_override;
+  bool album_override;
+  bool starred_album_override;
   sp_album *album;
   sp_artist *artist;
   sp_albumtype albumtype;
+  bool starred;
+  int compilation;
+  char *albumname;
+
+  spotify_cfg = cfg_getsec(cfg, "spotify");
+  artist_override = cfg_getbool(spotify_cfg, "artist_override");
+  starred_artist_override = cfg_getbool(spotify_cfg, "starred_artist_override");
+  album_override = cfg_getbool(spotify_cfg, "album_override");
+  starred_album_override = cfg_getbool(spotify_cfg, "starred_album_override");
 
   album = fptr_sp_track_album(track);
   if (!album)
@@ -467,25 +499,61 @@ spotify_metadata_get(sp_track *track, struct media_file_info *mfi)
     return -1;
 
   albumtype = fptr_sp_album_type(album);
+  starred = fptr_sp_track_is_starred(g_sess, track);
+
+  /*
+   * Treat album as compilation if one of the following conditions is true:
+   * - spotfy album type is compilation
+   * - artist_override in config is set to true and track is not part of the starred playlist
+   * - starred_artist_override in config is set to true and track is part of the starred playlist
+   */
+  compilation = ((albumtype == SP_ALBUMTYPE_COMPILATION)
+		  || (starred && starred_artist_override)
+		  || (!starred && artist_override));
+
+  if ((starred && starred_album_override)
+      || (!starred && album_override))
+    albumname = strdup(pltitle);
+  else
+    albumname = strdup(fptr_sp_album_name(album));
 
   mfi->title       = strdup(fptr_sp_track_name(track));
-  mfi->album       = strdup(fptr_sp_album_name(album));
+  mfi->album       = albumname;
   mfi->artist      = strdup(fptr_sp_artist_name(artist));
   mfi->year        = fptr_sp_album_year(album);
   mfi->song_length = fptr_sp_track_duration(track);
   mfi->track       = fptr_sp_track_index(track);
   mfi->disc        = fptr_sp_track_disc(track);
-  mfi->compilation = (albumtype == SP_ALBUMTYPE_COMPILATION);
+  mfi->compilation = compilation;
   mfi->artwork     = ARTWORK_SPOTIFY;
   mfi->type        = strdup("spotify");
   mfi->codectype   = strdup("wav");
   mfi->description = strdup("Spotify audio");
+  mfi->time_added  = time_added;
+
+  DPRINTF(E_SPAM, L_SPOTIFY, "Metadata for track:\n"
+      "Title:       %s\n"
+      "Album:       %s\n"
+      "Artist:      %s\n"
+      "Year:        %u\n"
+      "Track:       %u\n"
+      "Disc:        %u\n"
+      "Compilation: %d\n"
+      "Starred:     %d\n",
+      mfi->title,
+      mfi->album,
+      mfi->artist,
+      mfi->year,
+      mfi->track,
+      mfi->disc,
+      mfi->compilation,
+      starred);
 
   return 0;
 }
 
 static int
-spotify_track_save(int plid, sp_track *track)
+spotify_track_save(int plid, sp_track *track, const char *pltitle, int time_added)
 {
   struct media_file_info mfi;
   sp_link *link;
@@ -494,8 +562,8 @@ spotify_track_save(int plid, sp_track *track)
 
   if (!fptr_sp_track_is_loaded(track))
     {
-      DPRINTF(E_INFO, L_SPOTIFY, "Metadata for track not ready yet\n");
-      return 0;
+      DPRINTF(E_LOG, L_SPOTIFY, "Track appears to no longer have the proper status\n");
+      return -1;
     }
 
   if (fptr_sp_track_get_availability(g_sess, track) != SP_TRACK_AVAILABILITY_AVAILABLE)
@@ -528,7 +596,7 @@ spotify_track_save(int plid, sp_track *track)
 
   memset(&mfi, 0, sizeof(struct media_file_info));
 
-  ret = spotify_metadata_get(track, &mfi);
+  ret = spotify_metadata_get(track, &mfi, pltitle, time_added);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Metadata missing (but track should be loaded?): '%s'\n", fptr_sp_track_name(track));
@@ -551,21 +619,38 @@ spotify_playlist_save(sp_playlist *pl)
   sp_link *link;
   char url[1024];
   const char *name;
-  char title[512];
   int plid;
   int num_tracks;
+  char virtual_path[PATH_MAX];
+  int time;
   int ret;
   int i;
   
   if (!fptr_sp_playlist_is_loaded(pl))
     {
-      DPRINTF(E_DBG, L_SPOTIFY, "Playlist still not loaded - wait for rename callback\n");
+      DPRINTF(E_DBG, L_SPOTIFY, "Playlist still not loaded - will wait for next callback\n");
       return 0;
     }
 
   name = fptr_sp_playlist_name(pl);
+  num_tracks = fptr_sp_playlist_num_tracks(pl);
 
-  DPRINTF(E_INFO, L_SPOTIFY, "Saving playlist: '%s'\n", name);
+  // The starred playlist has an empty name, set it manually to "Starred"
+  if (*name == '\0')
+    name = "Starred";
+
+  for (i = 0; i < num_tracks; i++)
+    {
+      track = fptr_sp_playlist_track(pl, i);
+
+      if (track && !fptr_sp_track_is_loaded(track))
+	{
+	  DPRINTF(E_DBG, L_SPOTIFY, "All playlist tracks not loaded (will wait for next callback): %s\n", name);
+	  return 0;
+	}
+    }
+
+  DPRINTF(E_LOG, L_SPOTIFY, "Saving playlist (%d tracks): '%s'\n", num_tracks, name);
 
   /* Save playlist (playlists table) */
   link = fptr_sp_link_create_from_playlist(pl);
@@ -582,10 +667,9 @@ spotify_playlist_save(sp_playlist *pl)
     }
   fptr_sp_link_release(link);
 
-//  sleep(1); // Primitive way of preventing database locking (the mutex wasn't working)
-
   pli = db_pl_fetch_bypath(url);
-  snprintf(title, sizeof(title), "[s] %s", name);
+
+  snprintf(virtual_path, PATH_MAX, "/spotify:/%s", name);
 
   if (pli)
     {
@@ -593,32 +677,56 @@ spotify_playlist_save(sp_playlist *pl)
 
       plid = pli->id;
 
-      free_pli(pli, 0);
+      free(pli->title);
+      pli->title = strdup(name);
+      free(pli->virtual_path);
+      pli->virtual_path = strdup(virtual_path);
 
-      ret = db_pl_update(title, url, plid);
+      ret = db_pl_update(pli);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error updating playlist ('%s', link %s)\n", name, url);
+
+	  free_pli(pli, 0);
 	  return -1;
 	}
 
-      db_pl_ping(plid);
       db_pl_clear_items(plid);
     }
   else
     {
       DPRINTF(E_DBG, L_SPOTIFY, "Adding playlist ('%s', link %s)\n", name, url);
 
-      ret = db_pl_add(title, url, &plid);
+      pli = (struct playlist_info *)malloc(sizeof(struct playlist_info));
+      if (!pli)
+	{
+	  DPRINTF(E_LOG, L_SCAN, "Out of memory\n");
+
+	  return -1;
+	}
+
+      memset(pli, 0, sizeof(struct playlist_info));
+
+      pli->type = PL_PLAIN;
+      pli->title = strdup(name);
+      pli->path = strdup(url);
+      pli->virtual_path = strdup(virtual_path);
+      pli->parent_id = g_base_plid;
+
+      ret = db_pl_add(pli, &plid);
       if ((ret < 0) || (plid < 1))
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error adding playlist ('%s', link %s, ret %d, plid %d)\n", name, url, ret, plid);
+
+	  free_pli(pli, 0);
 	  return -1;
 	}
     }
 
+  free_pli(pli, 0);
+
   /* Save tracks and playlistitems (files and playlistitems table) */
-  num_tracks = fptr_sp_playlist_num_tracks(pl);
+  db_transaction_begin();
   for (i = 0; i < num_tracks; i++)
     {
       track = fptr_sp_playlist_track(pl, i);
@@ -628,13 +736,16 @@ spotify_playlist_save(sp_playlist *pl)
 	  continue;
 	}
 
-      ret = spotify_track_save(plid, track);
+      time = fptr_sp_playlist_track_create_time(pl, i);
+
+      ret = spotify_track_save(plid, track, name, time);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error saving track %d to playlist '%s' (id %d)\n", i, name, plid);
 	  continue;
 	}
     }
+  db_transaction_end();
 
   return plid;
 }
@@ -951,6 +1062,7 @@ audio_get(struct spotify_command *cmd)
   struct timespec ts;
   audio_fifo_data_t *afd;
   int processed;
+  int timeout;
   int ret;
   int s;
 
@@ -975,9 +1087,11 @@ audio_get(struct spotify_command *cmd)
 
       // If buffer is empty, wait for audio, but use timed wait so we don't
       // risk waiting forever (maybe the player stopped while we were waiting)
+      timeout = 0;
       while ( !(afd = TAILQ_FIRST(&g_audio_fifo->q)) && 
 	       (g_state != SPOTIFY_STATE_STOPPED) &&
-	       (g_state != SPOTIFY_STATE_STOPPING) )
+	       (g_state != SPOTIFY_STATE_STOPPING) &&
+	       (timeout < SPOTIFY_TIMEOUT) )
 	{
 	  DPRINTF(E_DBG, L_SPOTIFY, "Waiting for audio\n");
 #if _POSIX_TIMERS > 0
@@ -988,10 +1102,18 @@ audio_get(struct spotify_command *cmd)
 	  TIMEVAL_TO_TIMESPEC(&tv, &ts);
 #endif
 	  ts.tv_sec += 5;
+	  timeout += 5;
 
 	  pthread_cond_timedwait(&g_audio_fifo->cond, &g_audio_fifo->mutex, &ts);
 	}
 
+      if ((!afd) && (timeout >= SPOTIFY_TIMEOUT))
+	{
+	  DPRINTF(E_LOG, L_SPOTIFY, "Timeout waiting for audio (waited %d sec)\n", timeout);
+
+	  spotify_playback_stop_nonblock();
+	}
+
       if (!afd)
 	break;
 
@@ -1058,11 +1180,11 @@ artwork_get(struct spotify_command *cmd)
       goto level2_exit;
     }
 
-  // TODO rescale
+  // Get an image at least the same size as requested
   image_size = SP_IMAGE_SIZE_SMALL; // 64x64
-  if ((cmd->arg.artwork.max_w > 200) && (cmd->arg.artwork.max_h > 200))
+  if ((cmd->arg.artwork.max_w > 64) || (cmd->arg.artwork.max_h > 64))
     image_size = SP_IMAGE_SIZE_NORMAL; // 300x300
-  if ((cmd->arg.artwork.max_w > 500) && (cmd->arg.artwork.max_h > 500))
+  if ((cmd->arg.artwork.max_w > 300) || (cmd->arg.artwork.max_h > 300))
     image_size = SP_IMAGE_SIZE_LARGE; // 640x640
 
   image_id = fptr_sp_album_cover(album, image_size);
@@ -1144,14 +1266,16 @@ artwork_get(struct spotify_command *cmd)
 static void
 logged_in(sp_session *sess, sp_error error)
 {
+  cfg_t *spotify_cfg;
   sp_playlist *pl;
   sp_playlistcontainer *pc;
+  struct playlist_info pli;
+  int ret;
   int i;
 
   if (SP_ERROR_OK != error)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Login failed: %s\n",	fptr_sp_error_message(error));
-      thread_exit();
       return;
     }
 
@@ -1159,6 +1283,27 @@ logged_in(sp_session *sess, sp_error error)
 
   db_spotify_purge();
 
+  pl = fptr_sp_session_starred_create(sess);
+  fptr_sp_playlist_add_callbacks(pl, &pl_callbacks, NULL);
+
+  spotify_cfg = cfg_getsec(cfg, "spotify");
+  if (! cfg_getbool(spotify_cfg, "base_playlist_disable"))
+    {
+      memset(&pli, 0, sizeof(struct playlist_info));
+      pli.title = "Spotify";
+      pli.type = PL_FOLDER;
+      pli.path = "spotify:playlistfolder";
+
+      ret = db_pl_add(&pli, &g_base_plid);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_SPOTIFY, "Error adding base playlist\n");
+	  return;
+	}
+    }
+  else
+    g_base_plid = 0;
+
   pc = fptr_sp_session_playlistcontainer(sess);
 
   fptr_sp_playlistcontainer_add_callbacks(pc, &pc_callbacks, NULL);
@@ -1172,6 +1317,24 @@ logged_in(sp_session *sess, sp_error error)
     }
 }
 
+/**
+ * Called when logout has been processed.
+ * Either called explicitly if you initialize a logout operation, or implicitly
+ * if there is a permanent connection error
+ *
+ * @sa sp_session_callbacks#logged_out
+ */
+static void
+logged_out(sp_session *sess)
+{
+  DPRINTF(E_INFO, L_SPOTIFY, "Logout complete\n");
+
+  pthread_mutex_lock(&login_lck);
+
+  pthread_cond_signal(&login_cond);
+  pthread_mutex_unlock(&login_lck);
+}
+
 /**
  * This callback is used from libspotify whenever there is PCM data available.
  *
@@ -1316,6 +1479,7 @@ static void end_of_track(sp_session *sess)
  */
 static sp_session_callbacks session_callbacks = {
   .logged_in = &logged_in,
+  .logged_out = &logged_out,
   .connectionstate_updated = &connectionstate_updated,
   .notify_main_thread = &notify_main_thread,
   .music_delivery = &music_delivery,
@@ -1556,7 +1720,7 @@ spotify_playback_stop(void)
   return ret;
 }
 
-/* Thread: libspotify */
+/* Thread: player and libspotify */
 void
 spotify_playback_stop_nonblock(void)
 {
@@ -1751,45 +1915,54 @@ spotify_file_read(char *path, char **username, char **password)
 void
 spotify_login(char *path)
 {
+  sp_error err;
   char *username;
   char *password;
   int ret;
-  sp_error err;
 
   if (!g_sess)
     {
-      DPRINTF(E_LOG, L_SPOTIFY, "Can't login! No valid Spotify session.\n");
+      if (!g_libhandle)
+	DPRINTF(E_LOG, L_SPOTIFY, "Can't login! - could not find libspotify\n");
+      else
+	DPRINTF(E_LOG, L_SPOTIFY, "Can't login! - no valid Spotify session\n");
+
       return;
     }
 
-  /* Log out if thread already running */
-  if (g_state != SPOTIFY_STATE_INACTIVE)
+  if (SP_CONNECTION_STATE_LOGGED_IN == fptr_sp_session_connectionstate(g_sess))
     {
-      DPRINTF(E_LOG, L_SPOTIFY, "Existing login terminating (state %d)\n", g_state);
+      pthread_mutex_lock(&login_lck);
 
-      thread_exit();
+      DPRINTF(E_LOG, L_SPOTIFY, "Logging out of Spotify (current state is %d)\n", g_state);
 
-      ret = pthread_join(tid_spotify, NULL);
-      if (ret != 0)
+      fptr_sp_session_player_unload(g_sess);
+      err = fptr_sp_session_logout(g_sess);
+
+      if (SP_ERROR_OK != err)
 	{
-	  DPRINTF(E_FATAL, L_SPOTIFY, "Could not join Spotify thread: %s\n", strerror(errno));
+	  DPRINTF(E_LOG, L_SPOTIFY, "Could not logout of Spotify: %s\n", fptr_sp_error_message(err));
+	  pthread_mutex_unlock(&login_lck);
 	  return;
 	}
+
+      pthread_cond_wait(&login_cond, &login_lck);
+      pthread_mutex_unlock(&login_lck);
     }
 
-  /* Log in */
+  DPRINTF(E_INFO, L_SPOTIFY, "Logging into Spotify\n");
   if (path)
     {
       ret = spotify_file_read(path, &username, &password);
       if (ret < 0)
 	return;
 
-      DPRINTF(E_INFO, L_SPOTIFY, "Logging into Spotify\n");
       err = fptr_sp_session_login(g_sess, username, password, 1, NULL);
+      free(username);
+      free(password);
     }
   else
     {
-      DPRINTF(E_INFO, L_SPOTIFY, "Logging into Spotify\n");
       err = fptr_sp_session_relogin(g_sess);
     }
 
@@ -1798,18 +1971,8 @@ spotify_login(char *path)
       DPRINTF(E_LOG, L_SPOTIFY, "Could not login into Spotify: %s\n", fptr_sp_error_message(err));
       return;
     }
-
-  /* Spawn thread */
-  ret = pthread_create(&tid_spotify, NULL, spotify, NULL);
-  if (ret < 0)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not spawn Spotify thread: %s\n", strerror(errno));
-
-      return;
-    }
 }
 
-
 /* Thread: main */
 int
 spotify_init(void)
@@ -1967,9 +2130,28 @@ spotify_init(void)
   pthread_mutex_init(&g_audio_fifo->mutex, NULL);
   pthread_cond_init(&g_audio_fifo->cond, NULL);
 
+  pthread_mutex_init(&login_lck, NULL);
+  pthread_cond_init(&login_cond, NULL);
+
+  /* Spawn thread */
+  ret = pthread_create(&tid_spotify, NULL, spotify, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_FATAL, L_SPOTIFY, "Could not spawn Spotify thread: %s\n", strerror(errno));
+      goto thread_fail;
+    }
+
   DPRINTF(E_DBG, L_SPOTIFY, "Spotify init complete\n");
   return 0;
 
+ thread_fail:
+  pthread_cond_destroy(&login_cond);
+  pthread_mutex_destroy(&login_lck);
+
+  pthread_cond_destroy(&g_audio_fifo->cond);
+  pthread_mutex_destroy(&g_audio_fifo->mutex);
+  free(g_audio_fifo);  
+
  audio_fifo_fail:
   fptr_sp_session_release(g_sess);
   g_sess = NULL;
@@ -2035,6 +2217,10 @@ spotify_deinit(void)
   close(g_exit_pipe[0]);
   close(g_exit_pipe[1]);
 
+  /* Destroy locks */
+  pthread_cond_destroy(&login_cond);
+  pthread_mutex_destroy(&login_lck);
+
   /* Clear audio fifo */
   pthread_cond_destroy(&g_audio_fifo->cond);
   pthread_mutex_destroy(&g_audio_fifo->mutex);
