@@ -3,7 +3,12 @@
 #define __SPOTIFY_H__
 
 #include "db.h"
-#include <event.h>
+#ifdef HAVE_LIBEVENT2
+# include <event2/event.h>
+# include <event2/buffer.h>
+#else
+# include <event.h>
+#endif
 
 int
 spotify_playback_play(struct media_file_info *mfi);
