@@ -7,11 +7,12 @@
 #else
 # include <event.h>
 #endif
+#include "http.h"
 
 struct transcode_ctx;
 
 int
-transcode(struct transcode_ctx *ctx, struct evbuffer *evbuf, int wanted);
+transcode(struct transcode_ctx *ctx, struct evbuffer *evbuf, int wanted, int *icy_timer);
 
 int
 transcode_seek(struct transcode_ctx *ctx, int ms);
@@ -25,4 +26,10 @@ transcode_cleanup(struct transcode_ctx *ctx);
 int
 transcode_needed(const char *user_agent, const char *client_codecs, char *file_codectype);
 
+void
+transcode_metadata(struct transcode_ctx *ctx, struct http_icy_metadata **metadata, int *changed);
+
+void
+transcode_metadata_artwork_url(struct transcode_ctx *ctx, char **artwork_url);
+
 #endif /* !__TRANSCODE_H__ */
